package bookranking;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.TimeZone;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.binary.Base64;
import org.springframework.integration.core.Message;
import org.springframework.integration.http.HttpHeaders;
import org.springframework.integration.message.MessageBuilder;
import org.springframework.integration.message.MessageSource;

/**
 * Amazonウェブサービス用リクエストを実行するためのメッセージアダプタ。
 */
public class AmazonWebAdapter implements MessageSource<Map<String, String>> {
    /** リクエストホスト名。 */
    private String host;
    /** リクエストパス。 */
    private String path;
    /** アクセスキー。 */
    private String accessKey;
    /** シークレットキー。 */
    private String secretKey;
    /** ノード。 */
    private String node;
    /** 検索インデックス。 */
    private String searchIndex;
    /** ソート順。 */
    private String sort;
    /**
     * リクエストホスト名を設定します。
     * @param newHost ホスト名
     */
    public void setHost(String newHost) {
        this.host = newHost;
    }
    /**
     * リクエストパスを設定します。
     * @param newPath パス
     */
    public void setPath(String newPath) {
        this.path = newPath;
    }
    /**
     * アクセスキーを設定します。
     * @param newAccessKey アクセスキー
     */
    public void setAccessKey(String newAccessKey) {
        this.accessKey = newAccessKey;
    }
    /**
     * シークレットキーを設定します。
     * @param newSecretKey シークレットキー
     */
    public void setSecretKey(String newSecretKey) {
        this.secretKey = newSecretKey;
    }
    /**
     * ノードを設定します。
     * @param newNode ノード
     */
    public void setNode(String newNode) {
        this.node = newNode;
    }
    /**
     * 検索インデックスを設定します。
     * @param newSearchIndex 検索インデックス
     */
    public void setSearchIndex(String newSearchIndex) {
        this.searchIndex = newSearchIndex;
    }
    /**
     * ソート順を設定します。
     * @param newSort ソート順
     */
    public void setSort(String newSort) {
        this.sort = newSort;
    }
    /**
     * {@inheritDoc}
     */
    @Override
    public Message<Map<String, String>> receive() {
        // Timestampパラメータ値を生成する
        String timestamp = createTimestampValue();
        // リクエストパラメータのマップを生成する。
        Map<String, String> paramMap = new LinkedHashMap<String, String>();
        paramMap.put("AWSAccessKeyId", accessKey);
        paramMap.put("BrowseNode", node);
        paramMap.put("Operation", "ItemSearch");
        paramMap.put("ResponseGroup", "Medium");
        paramMap.put("SearchIndex", searchIndex);
        paramMap.put("Service", "AWSECommerceService");
        paramMap.put("Sort", sort);
        paramMap.put("Timestamp", timestamp);
        paramMap.put("Version", "2009-03-31");
        // Signatureパラメータ値を生成し、設定する
        String signSrc = "GET" + "\n" + host + "\n" + path + "\n" + canonicalize(paramMap);
        paramMap.put("Signature", hmac(signSrc));
        // メッセージを生成して戻す
        return MessageBuilder.withPayload(paramMap)
        .setHeader(HttpHeaders.REQUEST_URL, "http://" + host + path)
        .setHeader(HttpHeaders.REQUEST_METHOD, "GET")
        .build();
    }
    /**
     * 現在時刻のTimestampパラメータ値を生成し、戻します。
     * @return Timestampパラメータ値
     */
    private String createTimestampValue() {
        Calendar cal = Calendar.getInstance();
        DateFormat dfm = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
        dfm.setTimeZone(TimeZone.getTimeZone("GMT"));
        return dfm.format(cal.getTime());
    }
    /**
     * 指定した文字列からHMAC-SHA署名を生成し、戻します。
     * @param signSrc 署名対象文字列
     * @return 署名文字列
     */
    private String hmac(String stringToSign) {
        String signature = null;
        try {
            // メッセージ認証コードオブジェクトを初期化する
            Mac mac = Mac.getInstance("HmacSHA256");
            SecretKeySpec secretKeySpec = new SecretKeySpec(secretKey.getBytes("UTF-8"), "HmacSHA256");
            mac.init(secretKeySpec);
            // 指定された文字列をUTF-8のバイト配列にする
            byte[] data = stringToSign.getBytes("UTF-8");
            // メッセージ認証コードを取得する
            byte[] rawHmac = mac.doFinal(data);
            // BASE64で文字列化する
            Base64 encoder = new Base64();
            signature = new String(encoder.encode(rawHmac));
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        } catch (InvalidKeyException e) {
            throw new RuntimeException(e);
        }
        return signature;
    }
    /**
     * 指定したパラメータマップを署名生成用に正規化した文字列を戻します。
     * @param paramMap パラメータマップ
     * @return 文字列
     */
    private String canonicalize(Map<String, String> paramMap) {
        // パラメータマップが空なら空文字列を戻す
        if (paramMap.isEmpty()) {
            return "";
        }
        // パラメータマップの内容を文字列化する
        StringBuffer buffer = new StringBuffer();
        boolean isFirst = true;
        for (Map.Entry<String, String> entry : paramMap.entrySet()) {
            // 各パラメータは'&'でつなぐ
            if (isFirst) {
                isFirst = false;
            } else {
                buffer.append("&");
            }
            // キー=値。ただし、パーセントエンコードする必要がある
            buffer.append(percentEncodeRfc3986(entry.getKey()));
            buffer.append("=");
            buffer.append(percentEncodeRfc3986(entry.getValue()));
        }
        String cannoical = buffer.toString();
        return cannoical;
    }
    /**
     * 指定した文字列をパーセントエンコードし、戻します。
     * @param s 文字列
     * @return パーセントエンコードした文字列
     */
    private String percentEncodeRfc3986(String s) {
        String out;
        try {
            out = URLEncoder.encode(s, "UTF-8")
                .replace("+", "%20")
                .replace("*", "%2A")
                .replace("%7E", "~");
        } catch (UnsupportedEncodingException e) {
            out = s;
        }
        return out;
    }
}
