package bookranking;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.springframework.integration.core.Message;
import org.springframework.integration.message.MessageBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Amazonウェブサービスのレスポンスを書籍一覧項目メッセージに変換するトランスフォーマ。
 */
public class AmazonWebResponseBookItemTransformer {

    /**
     * 指定したメッセージの内容をBookItemメッセージに変換して戻します。
     * @param srcMessage メッセージ
     * @return BookItemメッセージ
     */
    public Message<List<BookItem>> transform(Message<?> srcMessage) {
        List<BookItem> bookItemList = new ArrayList<BookItem>();
        // 指定されたメッセージの内容からDOMドキュメントオブジェクトを生成する
        String src = (String) srcMessage.getPayload();
        Document doc = convertToDocument(src);
        // 商品エレメントのリストを取得する
        NodeList itemList = doc.getElementsByTagName("Item");
        // 商品エレメントからBookItemオブジェクトを生成し、リストに追加する
        for (int i = 0, iEnd = itemList.getLength(); i < iEnd; i++) {
            BookItem bookItem = createBookItem((Element) itemList.item(i));
            bookItemList.add(bookItem);
        }
        // メッセージを生成し、戻す
        Message<List<BookItem>> bookItemListMessage = MessageBuilder.withPayload(bookItemList).copyHeaders(srcMessage.getHeaders()).build();
        return bookItemListMessage;
    }
    /**
     * 指定した商品ノードからBookItemオブジェクトを生成し、戻します。
     * @param item 商品ノード
     * @return BookItemオブジェクト
     */
    private BookItem createBookItem(Element item) {
        BookItem bookItem = new BookItem();
        NodeList childList = item.getChildNodes();
        for (int i = 0, iEnd = childList.getLength(); i < iEnd; i++) {
            Element child = (Element) childList.item(i);
            if ("DetailPageURL".equals(child.getTagName())) {
                // DetailPageURLタグの内容が詳細ページのURL
                bookItem.setUrl(child.getTextContent());
            } else if ("ItemAttributes".equals(child.getTagName())) {
                // 著者名を取得、設定する
                bookItem.setAuthor(findTextContentByTagName(child, "Author"));
                // ISBNを取得、設定する
                bookItem.setIsbn(findTextContentByTagName(child, "ISBN"));
                // 出版社を取得、設定する
                bookItem.setPublisher(findTextContentByTagName(child, "Publisher"));
                // タイトルを取得、設定する
                bookItem.setTitle(findTextContentByTagName(child, "Title"));
                // 表示用価格を取得、設定する
                NodeList nodeList = child.getElementsByTagName("ListPrice");
                if (nodeList != null && nodeList.getLength() > 0) {
                    bookItem.setDisplayPrice(findTextContentByTagName((Element) nodeList.item(0), "FormattedPrice"));
                }
            } else if ("LargeImage".equals(child.getTagName())) {
                // 画像URLを取得、設定する
                bookItem.setImageUrl(findTextContentByTagName(child, "URL"));
            }
        }
        return bookItem;
    }
    /**
     * 指定したエレメントから、指定したタグ名のエレメントの内容文字列を検索し、結果を戻します。
     * @param element エレメント
     * @param tagName タグ名
     * @return 内容文字列
     */
    String findTextContentByTagName(Element element, String tagName) {
        NodeList nodeList = element.getElementsByTagName(tagName);
        if (nodeList == null || nodeList.getLength() == 0) {
            return null;
        }
        return nodeList.item(0).getTextContent();
    }
    /**
     * 指定したXML文字列からDOMドキュメントオブジェクトを生成し、戻します。
     * @param src XML文字列
     * @return DOMドキュメントオブジェクト
     */
    private Document convertToDocument(String src) {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        try {
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document doc = builder.parse(new InputSource(new StringReader(src)));
            return doc;
        } catch (SAXException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        } catch (ParserConfigurationException e) {
            throw new RuntimeException(e);
        }
    }
}
